""" Tut2: Working with layers """

import tilengine as tln
from ctypes import c_int
from math import sin,radians

WIDTH = 420
HEIGHT = 240

# variables
class Layer:
	tileset = None
	tilemap = None
	palette = None

# load layer assets and basic setup
def load_layer (index, base_name):
	layer = Layer()
	layer.tileset = tln.LoadTileset (base_name + ".tsx")
	layer.tilemap = tln.LoadTilemap (base_name + ".tmx", None)
	layer.palette = tln.GetTilesetPalette (layer.tileset)
	tln.SetLayer (index, layer.tileset, layer.tilemap)
	return layer

# initialise
tln.Init (WIDTH,HEIGHT,3,0,0)
tln.CreateWindow (None, tln.CWF_VSYNC)

# setup the layers
load_layer (0, "Foreground")
load_layer (1, "Background")
load_layer (2, "Base")

# create column offset array
num_columns = WIDTH/8 + 2
columns = (c_int*num_columns)()
tln.SetLayerColumnOffset (0, columns)
tln.EnableBlur(True)

# game loop
x_world = 0
while tln.ProcessWindow ():
	for n in range(num_columns):
		columns[n] = int(sin(radians(x_world*4 + n*7)) * ((abs(sin(radians(x_world/4))*12)) + 4))
	tln.SetLayerPosition (0, x_world, 0)
	tln.SetLayerPosition (1, x_world/6, 0)
	tln.DrawFrame (0)
	x_world += 2
	
# release resources
tln.Deinit ()
