import tilengine as tln
from math import tan,radians

# helper constants
LAYER_SKY 	 = 0
LAYER_CLOUDS = 1
LAYER_LAND 	 = 2
WIDTH 		 = 360
HEIGHT 		 = 400
SKY_HEIGHT 	 = 64
LAND_HEIGHT  = HEIGHT - SKY_HEIGHT

# variables
rows = range(LAND_HEIGHT)
world_y = 6144 - 240

# load layer
def load_layer (index, base_name):
	tileset = tln.LoadTileset (base_name + ".tsx")
	tilemap = tln.LoadTilemap (base_name + ".tmx", None)
	tln.SetLayer (index, tileset, tilemap)	

# linear interpolation
def lerp (x, x0, x1, fx0, fx1):
	return fx0 + (fx1 - fx0)*(x - x0)/(x1 - x0)
	
# raster effect function
def raster_effect (line):
	if line >= SKY_HEIGHT:
		line -= SKY_HEIGHT
		tln.SetLayerPosition (LAYER_CLOUDS, 0,world_y*2 + rows[line] - line)
		tln.SetLayerPosition (LAYER_LAND, 0,world_y/2 + rows[line] - line)

# init
tln.Init (WIDTH,HEIGHT, 3,0,0)
tln.CreateWindow (None, tln.CWF_VSYNC)
tln.EnableBlur (True)
tln.SetBGColor (0,0,0)

# setup layers
load_layer (LAYER_SKY, "sky")
load_layer (LAYER_CLOUDS, "clouds")
load_layer (LAYER_LAND, "zelda")
tln.SetLayerClip (LAYER_SKY, 0,0, WIDTH,SKY_HEIGHT + 63)
tln.SetLayerClip (LAYER_CLOUDS, 0,SKY_HEIGHT, WIDTH,HEIGHT)
tln.SetLayerClip (LAYER_LAND, 0,SKY_HEIGHT, WIDTH,HEIGHT)
tln.SetLayerBlendMode (LAYER_SKY, tln.BLEND_ADD, 255)

# setup raster callback
my_raster_callback = tln.RasterCallbackFunc(raster_effect)
tln.SetRasterCallback (my_raster_callback)

# precalc raster effect values and store in rows table
for n in rows:
	angle = lerp(n, 0,LAND_HEIGHT, 105.0,180.0)
	rows[n] = int(tan(radians(angle)) * 240)

# main loop
while tln.ProcessWindow ():
	tln.DrawFrame (0)
	world_y -= 1

tln.Deinit ()
